package com.sysu.admin.utils.shape.postgis;

import java.util.HashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;

public class Main {
    public static void main(String[] args) {
        PostGisUtilTest postGisUtilTest = new PostGisUtilTest();
//        postGisUtilTest.testInsertShp();
//        try {
//            postGisUtilTest.createTable();
//        } catch (Exception e) {
//            e.printStackTrace();
//        }

//        postGisUtilTest.layers();
//        postGisUtilTest.updateCenterPoint();
//        postGisUtilTest.updateSrid();
//        postGisUtilTest.ST_Extent();
//        postGisUtilTest.createIndex();
//        postGisUtilTest.createCOLUMN();
        postGisUtilTest.removeTables();

    }
}
